#include "ecdsa.h"

void ecdsa_sign(
	EC *ec,						/* Elliptic curve paramaters */
	gmp_randstate_t state,		/* PRNG state, initialized by caller */
	mpz_t d,					/* Private key, 1 <= d < ec->n */
	mpz_t e,					/* Message */
	mpz_t r,					/* Signature */
	mpz_t s) {					/* Signature */

	ECP Q;
	mpz_t k, t0;

	mpz_init(Q.x);
	mpz_init(Q.y);
	mpz_init(k);
	mpz_init(t0);

	for(;;) {
		mpz_urandomm(k, state, ec->n);
		if(mpz_cmp_ui(k, 0) == 0) continue;
		ec_mul(ec, &Q, &ec->G, k);
		mpz_mod(r, Q.x, ec->n);
		if(mpz_cmp_ui(r, 0) == 0) continue;
		mpz_invert(k, k, ec->n);
		mpz_mul(t0, r, d);
		mpz_add(t0, t0, e);
		mpz_mod(t0, t0, ec->n);
		mpz_mul(t0, t0, k);
		mpz_mod(s, t0, ec->n);
		if(mpz_cmp_ui(s, 0) == 0) continue;
		break;
	}

	mpz_clear(Q.x);
	mpz_clear(Q.y);
	mpz_clear(k);
	mpz_clear(t0);
}

int ecdsa_verify(				/* Is the signature correct? */
	EC *ec,						/* Elliptic curve parameters */
	ECP *Q,						/* Public key (expanded) */
	mpz_t e,					/* Message */
	mpz_t r,					/* Signature */
	mpz_t s) {					/* Signature */

	mpz_t w, u1, u2;
	ECP P, R;
	int retval;

	if(mpz_cmp_ui(r, 1) < 0 || mpz_cmp(r, ec->n) >= 0) return 0;
	if(mpz_cmp_ui(s, 1) < 0 || mpz_cmp(s, ec->n) >= 0) return 0;

	mpz_init(w);
	mpz_init(u1);
	mpz_init(u2);
	mpz_init(P.x);
	mpz_init(P.y);
	mpz_init(R.x);
	mpz_init(R.y);

	mpz_invert(w, s, ec->n);

	mpz_mul(u1, e, w);
	mpz_mod(u1, u1, ec->n);

	mpz_mul(u2, r, w);
	mpz_mod(u2, u2, ec->n);

	ec_mul(ec, &P, &ec->G, u1);
	ec_mul(ec, &R, Q, u2);
	ec_add(ec, &P, &P, &R);

	retval = (mpz_cmp(P.x, r) == 0);

	mpz_clear(w);
	mpz_clear(u1);
	mpz_clear(u2);
	mpz_clear(P.x);
	mpz_clear(P.y);
	mpz_clear(R.x);
	mpz_clear(R.y);

	return retval;
}

